<?php
namespace Geral\Repositories;

class Prodfili extends \Doctrine\ORM\EntityRepository
{

    CONST P_STATUS_S = 'S';
    CONST P_STATUS_A = 'A';


    /**
     * Consulta os produtos de uma determinada filial com controle de estoque.
     *
     * @param $filial
     * @param $nrorg
     *
     * @return array
     */
    public function consultaProdutosFilial($filial, $nrorg){
        $sql = "SELECT P.*
                 FROM PRODUTO P, PRODFILI PR
                WHERE PR.CDFILIAL = :P_FILIAL
                  AND PR.NRORG    = :P_NRORG
                  AND PR.IDCNTRESTOQ = :P_STATUS_S
                  AND PR.IDPRODATIV  = :P_STATUS_A
                  AND PR.NRORG    = P.NRORG
                  AND P.CDPRODUTO = PR.CDPRODUTO
                  AND P.CDPRODUTO = P.CDPRODESTO
                  AND P.IDPRODATIVPR = :P_STATUS_S";

        $params = array(
            'P_FILIAL' => $filial,
            'P_NRORG' => $nrorg,
            'P_STATUS_S' => self::P_STATUS_S,
            'P_STATUS_A' => self::P_STATUS_A
        );

        $entityManager = $this->getEntityManager();
        $resultSetMapping = new \Doctrine\ORM\Query\ResultSetMappingBuilder($entityManager);
        $resultSetMapping->addRootEntityFromClassMetadata('Geral\Entities\Produto', "P");

        $result = $entityManager->createNativeQuery($sql, $resultSetMapping)->execute($params);
        return is_array($result) ? $result : false;
    }


    /**
     * Consulta os produtos de acordo com a posição do estoque no dia e com a filial.
     *
     * @param $nrorg
     * @param $filial
     * @param $dtPosicao
     * @param $produto
     * @param $almox
     * @param $local
     *
     * @return array
     */
    public function consultaDadosProdutoFilial($nrorg, $filial, $dtPosicao, $produto, $almox, $local){
        $sql = "SELECT D.NRORG, D.CDPRODUTO, P.NMPRODUTO, F.IDCNTRESTOQ, P.CDPRODESTO, D.NRLOTEESTQ,
                       D.NRSUBLOTE, D.QTESTOQDIA
                  FROM PRODUTO P,
                       POSESTDIA D,
                       PRODFILI F,
                       PRODFILI C,
                       (SELECT NRORG, CDFILIAL, CDALMOXARIFE, CDLOCALESTOQ, CDPRODUTO, NRLOTEESTQ,
                               NRSUBLOTE, MAX(DTPOSIESTQ) AS DTPOSIESTQ
                          FROM POSESTDIA
                         WHERE DTPOSIESTQ <= :P_DTPOSICAO
                         GROUP BY NRORG, CDFILIAL, CDALMOXARIFE, CDLOCALESTOQ, CDPRODUTO, NRLOTEESTQ,
                               NRSUBLOTE) A
                 WHERE P.CDPRODUTO    = :P_CDPRODUTO
                   AND P.NRORG        = :P_NRORG
                   AND P.IDPRODATIVPR = :P_STATUS_S
                   AND F.NRORG        = P.NRORG
                   AND F.CDPRODUTO    = P.CDPRODUTO
                   AND F.CDFILIAL     = :P_CDFILIAL
                   AND F.IDCNTRESTOQ  = :P_STATUS_S
                   AND F.IDPRODATIV   = :P_STATUS_A
                   AND F.NRORG        = C.NRORG
                   AND F.CDFILIAL     = C.CDFILIAL
                   AND F.CDPRODUTO    = D.CDPRODUTO
                   AND C.CDFILIAL     = D.CDFILIAL
                   AND C.NRORG        = D.NRORG
                   AND C.CDPRODUTO    = D.CDPRODUTO
                   AND C.IDPRODATIV   = :P_STATUS_A
                   AND D.CDFILIAL     = :P_CDFILIAL
                   AND D.CDALMOXARIFE = :P_CDALMOXARIFE
                   AND D.CDLOCALESTOQ = :P_CDLOCALESTOQ
                   AND D.CDPRODUTO    = P.CDPRODUTO
                   AND D.QTESTOQDIA   <> 0
                   AND D.CDFILIAL     = A.CDFILIAL
                   AND D.NRORG        = A.NRORG
                   AND D.CDALMOXARIFE = A.CDALMOXARIFE
                   AND D.CDLOCALESTOQ = A.CDLOCALESTOQ
                   AND D.CDPRODUTO    = A.CDPRODUTO
                   AND D.NRLOTEESTQ   = A.NRLOTEESTQ
                   AND D.NRSUBLOTE    = A.NRSUBLOTE
                   AND D.DTPOSIESTQ   = A.DTPOSIESTQ";

        $params = array(
            'P_DTPOSICAO' => $dtPosicao,
            'P_CDPRODUTO' => $produto,
            'P_CDFILIAL' => $filial,
            'P_NRORG' => $nrorg,
            'P_STATUS_S' => self::P_STATUS_S,
            'P_STATUS_A' => self::P_STATUS_A,
            'P_CDALMOXARIFE' => $almox,
            'P_CDLOCALESTOQ' => $local,
        );

        $result = $this->_em->getConnection()->fetchAll($sql, $params);
        return is_array($result) ? current($result) : false;
    }


    public function consultaProdutosDtPosicao($nrorg, $filial, $dtPosicao, $cdAlmox, $cdLocal){
        $sql = "SELECT DISTINCT D.CDPRODUTO, P.CDPRODESTO, D.QTESTOQDIA, D.CDALMOXARIFE, D.CDLOCALESTOQ
                  FROM PRODUTO P, POSESTDIA D, PRODFILI F, PRODFILI C,
                       (SELECT CDFILIAL, CDALMOXARIFE, CDLOCALESTOQ, CDPRODUTO, NRLOTEESTQ,
                               NRSUBLOTE, MAX(DTPOSIESTQ) AS DTPOSIESTQ, NRORG
                          FROM POSESTDIA
                         WHERE DTPOSIESTQ <= :P_DTPOSICAO
                         GROUP BY CDFILIAL, CDALMOXARIFE, CDLOCALESTOQ, CDPRODUTO, NRLOTEESTQ, NRSUBLOTE, NRORG) AX
                 WHERE D.CDFILIAL     = :P_CDFILIAL
                   AND D.NRORG        = :P_NRORG
                   AND D.CDALMOXARIFE = :P_CDALMOXARIFE
                   AND D.CDLOCALESTOQ = :P_CDLOCALESTOQ
                   AND D.NRORG        = P.NRORG
                   AND D.CDPRODUTO    = P.CDPRODUTO
                   AND P.IDPRODATIVPR = :P_STATUS_S
                   AND P.CDPRODESTO   = F.CDPRODUTO
                   AND P.NRORG        = F.NRORG
                   AND F.IDCNTRESTOQ  = :P_STATUS_S
                   AND F.IDPRODATIV   = :P_STATUS_A
                   AND F.CDFILIAL     = D.CDFILIAL
                   AND F.NRORG        = D.NRORG
                   AND D.CDFILIAL     = AX.CDFILIAL
                   AND D.CDALMOXARIFE = AX.CDALMOXARIFE
                   AND D.CDLOCALESTOQ = AX.CDLOCALESTOQ
                   AND D.CDPRODUTO    = AX.CDPRODUTO
                   AND D.NRLOTEESTQ   = AX.NRLOTEESTQ
                   AND D.NRSUBLOTE    = AX.NRSUBLOTE
                   AND D.DTPOSIESTQ   = AX.DTPOSIESTQ
                   AND D.NRORG        = AX.NRORG
                   AND C.NRORG        = D.NRORG
                   AND C.CDFILIAL     = D.CDFILIAL
                   AND C.CDPRODUTO    = D.CDPRODUTO
                   AND C.IDPRODATIV   = :P_STATUS_A";

        $params = array(
            'P_DTPOSICAO' => $dtPosicao,
            'P_CDFILIAL' => $filial,
            'P_CDALMOXARIFE' => $cdAlmox,
            'P_CDLOCALESTOQ' => $cdLocal,
            'P_NRORG' => $nrorg,
            'P_STATUS_S' => self::P_STATUS_S,
            'P_STATUS_A' => self::P_STATUS_A,
        );

        $result = $this->_em->getConnection()->fetchAll($sql, $params);
        return count($result)>0 ? $result : false;
    }


}